/* Copyright (C) 2004-2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include <string>
#include "agent/main.h"

/*
 * set Functions
 */

void Agent::setConfDir(std::string dir) {
    this->confDir = this->prefix + dir;
}

void Agent::setDefaultConf(std::string defaultConf) {
    this->defaultConf = this->prefix + defaultConf;
}

void Agent::setLogFile(std::string logDir, std::string file) {
    this->logFile = logDir + file;
}

void Agent::setPrefix(std::string p, int t) {
    this->prefix = getAbsolutePath(p, t);
}

void Agent::setVersion(std::string v) {
    this->version = v;
}

/*
 * get Functions
 */

std::string Agent::getConfDir() const {
    return this->confDir;
}

std::string Agent::getDefaultConf() const {
    return  this->defaultConf;
}

std::string Agent::getLogFile() const {
    return this->logFile;
}

std::string Agent::getPrefix() const {
    return this->prefix;
}

std::string Agent::getVersion() const {
    return this->version;
}

/*
 * getAbsolutePath Function:
 * get the absolute path of the agent
 */

std::string getAbsolutePath(std::string str, int tot) {
    int i = str.size(), count = 0;
        while (i-->-1)
        if (str[i] == '/')
            if (++count == tot)
                break;
    return str.substr(0, i);
}

/*
 * prefixIsSet Function
 * Check if the absolute path of agent was setted.
 */

bool prefixIsSet(std::string prefix) {
    if (prefix.size() == 0)
        throw " ERROR: Prefix not set. ";
    return true;
}

// NOLINTNEXTLINE(runtime/references)
int datasid_agent() {
    Agent agent;
    Conf conf;
    Proxy proxy;

    Json::Value update;

    /* Set agent prefix */
    agent.setPrefix(__FILE__, 4);

    /* Set confif file */
    agent.setDefaultConf("/conf/datasid-conf.json");

    /* Run datasid-common.cpp */
    if (datasid_common(update, &agent, &conf, &proxy)) {
        std::fstream logFile(agent.getLogFile().c_str(), std::fstream::app);
        logFile << __DATE__ << " - Agent started." << std::endl;
        logFile << __DATE__ << " - Start of update process execution."
                    << std::endl;
    }

    #ifdef UPDATE_JSON
            Json::StyledWriter styledWriter;
            if (!update.isNull())
                std::cout << styledWriter.write(update) <<  std::endl;
            else
                std::cout << "Empty updat object" << std::endl;
        #endif

    return 1;
}
