/* Copyright (C) 2004-2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include <string>
#include "agent/linux/datasid-common.h"

/*
 * fileExist Function
 * verify if a file exist
 */

bool fileExist(const std::string& file) {
    std::ifstream myFile(file.c_str(), std::ifstream::in);
    if (myFile.is_open()) {
        myFile.close();
        return true;
    }
    myFile.close();
    return false;
}

// NOLINTNEXTLINE(runtime/references)
bool datasid_common(Json::Value& inventory, Agent* agent, Conf* conf,
                    Proxy* proxy) {
    try {
        prefixIsSet(agent->getPrefix());
    } catch (const char* msg) {
        std::string var = __DATE__ + *msg;
        inventory["data"]["prefix_not_set"] =  var;
        return false;
    }

    /* Read config file and declare only valid variables */

    if (!fileExist(agent->getDefaultConf())) {
        std::string var = __DATE__ +
            std::string(" - ERROR: Conf file not found.");

        inventory["data"]["conf_file_not_found"] =  var;
        return false;
    } else {
       if (!datasid_conf(agent, conf, inventory)) {
            return false;
        }
    }

    /* Read proxy file and declare only valid variables */

    if (!fileExist(conf->getProxyConf())) {
        std::string var = __DATE__ +
            std::string(" - WARNING: Proxy file not found.");

        inventory["data"]["proxy_file_not_found"] =  var;
    } else {
        if (!datasid_proxy(conf, proxy, inventory)) {
            return false;
        }
    }

    return true;
}
