Requirements
------------

* docker

Docker
------

Docker supplies images of main Linux distributions from which one can build
their own with a single configuration file `Dockerfile`. It looks like a
virtual machine, but uses the kernel of the hosting OS.

Running
-------

    ./test.sh [-c] [-b [IMAGE_NAME]] [-r [IMAGE_NAME]]

The `test.sh` script, run without arguments, executes these three steps:

* copy the agent directories and files, plus other needed scripts, to a
directory inside each distribution directory - observation: this must be done
because docker does not allow inclusion of directories by relative paths.
* build/update all docker images - install requirements, build and compile the
agent, and run other needed scripts
* run all docker images - run the agent inside the images, print the JSON
generated in execution

Building the images and the agent may take some time and throw a lot of output,
so you can use the options

* `-c`              to copy (rsync) the agent into the distribution directories
* `-b [IMAGE_NAME]` to build the docker image specified by IMAGE_NAME or all
                    of them if none specified
* `-r [IMAGE_NAME]` to run the docker image specified by IMAGE_NAME or all
                    of them if none specified
* `-q`              to run in quiet mode: redirect stdout to log.txt during
                    steps "copy" and "build"

to execute only part of the flow. Of course, running will not work if the images
have not been built once previously.

Images
------

Currently tested images are

* ubuntu
* debian
* opensuse

Dockerfiles
-------------------

* Make use of Docker cache when trying stuff: avoid adding instructions
before long-standing ones if possible. (For example: once docker executes `RUN
apt-get update && apt-get install ...`, the resulting image is cached and
subsequent builds with no changes before that instruction will not execute it
again.)

* Refer to best practices
    * https://docs.docker.com/engine/userguide/eng-image/dockerfile_best-practices/
    * http://crosbymichael.com/dockerfile-best-practices-take-2.html
